@@ -55,7 +55,8 @@
 #' 
 #' @param A A 3D array (p x k x n) containing Procrustes shape variables for all specimens, or a matrix (n x variables)
 #' @param A2 An optional 3D array (p x k x n) containing Procrustes shape variables for all specimens, or a matrix (n x variables) for a second partition
-#' @param partition.gp A list of which landmarks (or variables) belong in which partition (e.g. A,A,A,B,B,B,C,C,C) (required when only 1 dataset provided)
+#' @param partition.gp A list of which landmarks (or variables) belong in which partition: 
+#' (e.g. A, A, A, B, B, B, C, C, C). Required when only 1 dataset provided.
 #' @param iter Number of iterations for significance testing
 #' @param seed An optional argument for setting the seed for random permutations of the resampling procedure.  
 #' If left NULL (the default), the exact same P-values will be found for repeated runs of the analysis (with the same number of iterations).
@@ -109,49 +110,103 @@
 #' plot(IT) # PLS plot
 #' 
 #'  
-#'  ### Visualize shape variation using picknplot.shape Because picknplot requires 
-#'  ### user decisions, the following example
+#'  ### Visualize shape variation using picknplot.shape Because picknplot  
+#'  ### requires user decisions, the following example
 #'  ### is not run (but can be with removal of #).
 #'  ### For detailed options, see the picknplot help file
 #'  # picknplot.shape(plot(IT))
 #' 
-#' IT$left.pls.vectors # extracting just the left (first block) singular vectors
+#' IT$left.pls.vectors # extracting just the left (first block) 
+#' # singular vectors
 
-integration.test<-function(A, A2=NULL,partition.gp=NULL,iter=999, seed=NULL, print.progress=TRUE){
-  if(any(is.na(A))==T){
-    stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').")  }
-  if(!is.null(partition.gp)){
-    partition.gp<-as.factor(partition.gp)
-    if (length(dim(A))==3){ x<-two.d.array(A)
-    p<-dim(A)[1]; k<-dim(A)[2];n<-dim(A)[3]
-    if(length(partition.gp)!=p){stop("Not all landmarks are assigned to a partition.")}
-    gps<-as.factor(rep(partition.gp,k,each = k, length=p*k))  
-    A.new<-A[which(partition.gp==levels(partition.gp)[1]),,]
-    A2.new<-A[which(partition.gp!=levels(partition.gp)[1]),,]
+integration.test <-function(A, A2 = NULL,
+                            partition.gp = NULL,
+                            iter = 999, seed = NULL, 
+                            print.progress = TRUE){
+  
+  if(any(is.na(A)))
+    stop("\nData matrix contains missing values. Estimate these first (see 'estimate.missing').",
+         call. = FALSE)  
+  
+  x <- try(two.d.array(A), silent = TRUE)
+  if(inherits(x, "try-error")) x <- try(as.matrix(A), silent = TRUE)
+  if(inherits(x, "try-error"))
+    stop("\nA is not a suitable data array for analysis. ", call. = FALSE)
+  
+  namesX <- rownames(x)
+  if (is.null(namesX)) namesX <- 1:NROW(x)
+  
+  if(!is.null(A2)) {
+    if(any(is.na(A2)))
+      stop("\nData matrix 2 contains missing values. Estimate these first (see 'estimate.missing').",
+           call. = FALSE) 
+    
+    y <- try(two.d.array(A2), silent = TRUE)
+    if(inherits(y, "try-error")) y <- try(as.matrix(A), silent = TRUE)
+    if(inherits(y, "try-error"))
+      stop("\nA2 is not a suitable data array for analysis. ", call. = FALSE)
+    
+    namesY <- rownames(y)
+    if (is.null(namesY)) {
+      namesY <- 1:NROW(y)
+      cat("\nNo names for A2.  Data are assumed to be ordered as in A.\n")
     }
-    if (length(dim(A))==2){ x<-A; A.new<-A
-    if(length(partition.gp)!=ncol(x)){stop("Not all variables are assigned to a partition.")}
-    gps<-as.factor(partition.gp) ;n<-dim(x)[2] 
-    A.new<-x[,which(gps==levels(gps)[1])]; A2.new<-x[,which(gps==levels(gps)[2])]
+    A1.new <- A
+    A2.new <- A2
+  }
+  
+  if(!is.null(partition.gp) && is.null(A2)){
+    partition.gp <- as.factor(partition.gp)
+    if (length(dim(A)) == 3){ 
+      dims <- dim(A)
+      p <- dims[1]
+      k <- dims[2]
+      
+      if(length(partition.gp) != p) 
+        stop("\nNot all landmarks are assigned to a partition.", call. = FALSE)
+      
+      gps <- as.factor(rep(partition.gp, k, each = k, length = p * k))  
+      A1.new <- A[which(partition.gp == levels(partition.gp)[1]),,]
+      A2.new <- A[which(partition.gp != levels(partition.gp)[1]),,]
     }
-    ngps<-nlevels(gps)
-    if(ngps==2){
-      y<-x[,which(gps==levels(gps)[2])]
-      x<-x[,which(gps==levels(gps)[1])]
+    
+    if (length(dim(A)) == 2){ 
+      
+      if(length(partition.gp) != ncol(x))
+        stop("\nNot all variables are assigned to a partition.", call. = FALSE)
+      
+      gps <- as.factor(partition.gp) 
+      A1.new <- A[, which(partition.gp == levels(partition.gp)[1])]
+      A2.new <- A[, which(partition.gp != levels(partition.gp)[1])]
+      
+    }
+    
+    ngps <- nlevels(gps)
+    
+    if(ngps == 2){
+      y <- x[,which(gps == levels(gps)[2])]
+      x <- x[,which(gps == levels(gps)[1])]
     }
   }
-  if(!is.null(seed) && seed=="random") seed = sample(1:iter, 1)
+  
+  
   if(!is.null(A2)){
-    A.new<-A; A2.new<-A2
-    if(any(is.na(A2))==T){
-      stop("Data matrix 2 contains missing values. Estimate these first (see 'estimate.missing').")  }
-    if (length(dim(A))==2){ x<-A }
-    if (length(dim(A))==3){ x<-two.d.array(A)}
-    if (length(dim(A2))==2){ y<-A2}
-    if (length(dim(A2))==3){ y<-two.d.array(A2)}
-    ngps=2; n<-dim(x)[2]
+    ngps <- 2
+    y <- as.matrix(y[match(namesX, namesY),])  
   }
-  if(ngps==2){
+  
+  n <- NROW(x)
+  
+  if(!is.null(seed) && seed == "random") seed = sample(1:iter, 1)
+  ind <- perm.index(n, iter, seed = seed)
+  perms <- length(ind)
+  
+  if(print.progress){
+    cat(paste("\nRandom PLS calculations:", perms, "permutations.\n"))
+    pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
+  }
+  
+  if(ngps == 2){
     pls.obs <- pls(x, y, verbose=TRUE)
     if(NCOL(x) > NROW(x)){
       pcax <- prcomp(x)
@@ -163,37 +218,81 @@ integration.test<-function(A, A2=NULL,partition.gp=NULL,iter=999, seed=NULL, pri
       d <- which(zapsmall(pcay$sdev) > 0)
       y <- pcay$x[,d]
     }
-    if(print.progress) pls.rand <- apply.pls(center(x), center(y), iter=iter, seed=seed) else
-      pls.rand <- .apply.pls(center(x), center(y), iter=iter, seed=seed)
+    
+    x <- center(x)
+    y <- center(x)
+    pls.rand <- sapply(1:perms, function(j) {
+      step <- j
+      if(print.progress) setTxtProgressBar(pb,step)
+      s <- ind[[j]]
+      quick.pls(x[s,], y)
+    })
+    
+    names(pls.rand) <- c("obs", paste("iter", 1:iter, sep = "."))
+    p.vals <- NULL
+    Zs <- NULL
     p.val <- pval(abs(pls.rand))
-    Z <- effect.size(abs(pls.rand), center=TRUE) 
+    Z <- effect.size(pls.rand, center=TRUE) 
     XScores <- pls.obs$XScores
     YScores <- pls.obs$YScores
   }
-  if(ngps>2){
-    pls.obs <- plsmulti(x, gps)  
-    if(print.progress) pls.rand <- apply.plsmulti(center(x), gps, iter=iter, seed=seed) else
-      pls.rand <- .apply.plsmulti(center(x), gps, iter=iter, seed=seed)
-    p.val <- pval(abs(pls.rand))
-    Z <- effect.size(abs(pls.rand), center=TRUE) 
+  
+  if(ngps > 2){
+    y <- center(x)
+    g <- factor(as.numeric(gps))
+    gps.combo <- combn(ngps, 2)
+    pls.rand <- sapply(1:perms, function(j) {
+      step <- j
+      if(print.progress) setTxtProgressBar(pb,step)
+      s <- ind[[j]]
+      sapply(1:ncol(gps.combo), function(jj) {
+        y1 <- as.matrix(y[s, g == gps.combo[1,jj]])
+        y2 <- y[ , g == gps.combo[2,jj]]
+        quick.pls(y1, y2)
+      })
+    })
+    
+    nms <- levels(gps)
+    rnms <- apply(gps.combo, 2, function(x) paste(nms[x[1]], nms[x[2]], sep = "-"))
+    cnms <- c("obs", paste("iter", 1:iter, sep = "."))
+    dimnames(pls.rand) <- list(rnms, cnms)
+    
+    p.vals <- apply(abs(pls.rand), 1, pval)
+    Zs <- apply(pls.rand, 1, effect.size)
+    p.val <- pval(colMeans(abs(pls.rand)))
+    Z <- effect.size(colMeans(pls.rand), center=TRUE)
+    r.pls.mat <- matrix(0, length(nms), length(nms))
+    dimnames(r.pls.mat) <- list(nms, nms)
+    r.pls.mat <- as.dist(r.pls.mat)
+    r.pls.mat[1:nrow(pls.rand)] <- pls.rand[, 1]
   }  
+  
+  step <- perms + 1
+  if(print.progress) {
+    setTxtProgressBar(pb,step)
+    cat("\n")
+    close(pb)
+  }
+  
   ####OUTPUT
-  if(ngps > 2) r.pls.mat <- pls.obs$r.pls.mat else r.pls.mat <- NULL
-  if(ngps==2){
-    out <- list(r.pls = pls.obs$r.pls, r.pls.mat = r.pls.mat, P.value = p.val, Z = Z,
+  
+  if(ngps == 2){
+    out <- list(r.pls = pls.obs$r.pls, r.pls.mat = NULL, P.value = p.val, Z = Z,
                 left.pls.vectors = pls.obs$left.vectors,
                 right.pls.vectors = pls.obs$right.vectors,
                 random.r = pls.rand, 
                 XScores = pls.obs$XScores,
                 YScores = pls.obs$YScores,
                 svd = pls.obs$pls.svd,
-                A1 = A.new, A2 = A2.new,
+                A1 = A1.new, A2 = A2.new,
                 A1.matrix = x, A2.matrix =y,
                 permutations = iter+1, call=match.call(),
                 method = "PLS")
   }
-  if(ngps>2){
-    out <- list(r.pls = pls.obs$r.pls, r.pls.mat = r.pls.mat, P.value = p.val, Z = Z,
+  if(ngps > 2){
+    out <- list(r.pls = mean(r.pls.mat), r.pls.mat = r.pls.mat, 
+                P.value = p.val, Z = Z,
+                pairwise.P.values = p.vals, pairwise.Z = Zs,
                 random.r = pls.rand, 
                 permutations = iter+1, call=match.call(),
                 method = "PLS")
